#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts: 
# Creation Date: 2021-03-14
# Language: Python3

################################### IMPORTS ####################################

# Standard library 
# Your imports from the standard library go here 


# External imports 
# Your imports from other packages go here 


# Internal imports 
from compass.ast.binary.binary import Binary
from compass.ast.binary.and_op import AndOp
from compass.ast.binary.div_op import DivOp
from compass.ast.binary.minus_op import MinusOp
from compass.ast.binary.mod_op import ModOp
from compass.ast.binary.or_op import OrOp
from compass.ast.binary.plus_op import PlusOp
from compass.ast.binary.testeq_op import TestEqOp
from compass.ast.binary.times_op import TimesOp

################################### CLASSES ####################################

# Your classes go here 

################################## FUNCTIONS ###################################

# Your functions go here 

##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################
